;=============================================================================
;	Filename:	midimultihold.asm
;=============================================================================
;	Author: 	Johannes Taelman
;	Company:	Johannes Taelman
;	Revision:	1.00
;	Date:		May 2006
;=============================================================================
; Specs:
;
; - midi input
; - 16 hold outputs 
; - every note has a setting for all outputs associated
; - for midiPulse rev3 pcb
; - for PIC18F2525 
;=============================================================================

 IFDEF __18F252
		list p=18f252		;list directive to define processor
		#include <p18f252.inc>	;processor specific definitions
 ENDIF
 IFDEF __18F2525
		list p=18f2525		;list directive to define processor
		#include <p18f2525.inc>	;processor specific definitions
 ENDIF

#define OUTPUT1  LATA,2; pin 4
#define OUTPUT2  LATA,1;
#define OUTPUT3  LATA,0; pin 2
#define OUTPUT4	 LATA,3; pin 5
#define OUTPUT5  LATA,4;
#define OUTPUT6  LATA,5; pin 7
#define OUTPUT7  LATB,7; pin 28
#define OUTPUT8  LATB,6;
#define OUTPUT9  LATB,5;
#define OUTPUT10 LATB,4;
#define OUTPUT11 LATB,3;
#define OUTPUT12 LATB,2;
#define OUTPUT13 LATB,1;
#define OUTPUT14 LATB,0; pin 21
#define OUTPUT15 LATC,5; pin 16
#define OUTPUT16 LATC,4; pin 15

#define	ErrorLed	LATC,	0;
#define SetErrorLed	BCF	ErrorLed
#define ClearErrorLed	BSF	ErrorLed
;#define MIDITHROUGHTEST

		#include "..\midi.asm"

; #define simulate 1 ; enable simulation
; #define heartbeat 1 ; enable heatbeats
;******************************************************************************
;Configuration bits
	CONFIG	OSC = HSPLL
	CONFIG	FCMEN = OFF
	CONFIG  IESO = OFF
;	CONFIG	PWRT = ON 
	CONFIG	PWRT = OFF
	CONFIG	BOREN = ON	
	CONFIG	BORV = 2	
	CONFIG	WDT = OFF
	CONFIG	WDTPS = 1
	CONFIG  MCLRE = ON
	CONFIG  LPT1OSC = OFF
	CONFIG  PBADEN = OFF
	CONFIG	CCP2MX = PORTC
	CONFIG	STVREN = ON
	CONFIG	LVP = OFF
	CONFIG  XINST = OFF
	CONFIG  DEBUG = OFF
;	CONFIG  DEBUG = ON

;----------------------------------------------------------------------------
;Constants

SPBRG_VAL	EQU	 04Fh		;set baud rate 31250 for 40Mhz clock

TX_BUF_LEN	EQU	010h		;length of transmit circular buffer
RX_BUF_LEN	EQU	TX_BUF_LEN	;length of receive circular buffer

;----------------------------------------------------------------------------
;Bit Definitions

TxBufFull	EQU	0		;bit indicates Tx buffer is full
TxBufEmpty	EQU	1		;bit indicates Tx buffer is empty
RxBufFull	EQU	2		;bit indicates Rx buffer is full
RxBufEmpty	EQU	3		;bit indicates Rx buffer is empty

;----------------------------------------------------------------------------
; Variables
; memory map 
;----------------------------------------------------------------------------

		CBLOCK	0x000
		WREG_TEMP		;to save WREG during interrupt
		STATUS_TEMP		;to save STATUS during interrupt
		BSR_TEMP		;to save BSR during interrupt
		FSR0H_TEMP		;to save FSR0H during interrupt
		FSR0L_TEMP		;to save FSR0L during interrupt
		FSR0H_SHADOW		;to save FSR0H during high interrupt
		FSR0L_SHADOW		;to save FSR0L during high interrupt
		TmpM			; non-ISR scratch
		TmpT			; non-ISR scratch		
		COUNTER
		COUNTER_HI
		CODE_ADDR_UPPER
		CODE_ADDR_HIGH
		CODE_ADDR_LOW
		BUFFER_ADDR_HIGH
		BUFFER_ADDR_LOW
		ENDC
		
;----------------------------------------------------------------------------
; end of memory map
;----------------------------------------------------------------------------


;----------------------------------------------------------------------------
;This code executes when a reset occurs.

		ORG     0x0000		;place code at reset vector

ResetVector:	
		bra	Main		;go to beginning of program

;----------------------------------------------------------------------------
;This code executes when a high priority interrupt occurs.

		ORG	0x0008

HighInt:
		bra	HighIntCode	;go to high priority interrupt routine

;----------------------------------------------------------------------------
;This code executes when a low priority interrupt occurs.

		ORG	0x0018

LowInt:	
		movff	STATUS,STATUS_TEMP	;save STATUS register
		movff	WREG,WREG_TEMP		;save working register
		movff	BSR,BSR_TEMP		;save BSR register
		movff	FSR0H,FSR0H_TEMP	;save FSR0H register
		movff	FSR0L,FSR0L_TEMP	;save FSR0L register

		;test other interrupt flags here

		btfss	PIR1,RCIF	;test for RCIF receive interrupt flag
		bra		LowInt1	;if RCIF is not set, done with test
		btfsc	PIE1,RCIE	;else test if Rx interrupt enabled
		bra		GetData		;if so, go get received data
LowInt1:
		btfss	PIR1,TXIF	;test for TXIF transmit interrupt flag
		bra		LowInt2		;if TXIF is not set, done with test
		btfsc	PIE1,TXIE	;else test if Tx interrupt is enabled
		bra		PutData		;if so, go transmit data

;can do special error handling here - an unexpected interrupt occurred 

LowInt2:
		reset			;error if no valid interrupt so reset
		
;------------------------------------
;End of low priority interrupt routine restores context.

EndLowInt:	
		movff	FSR0L_TEMP,FSR0L	;restore FSR0L register
		movff	FSR0H_TEMP,FSR0H	;restore FSR0H register
		movff	BSR_TEMP,BSR		;restore BSR register
		movff	WREG_TEMP,WREG		;restore working register
		movff	STATUS_TEMP,STATUS	;restore STATUS register
		retfie

;----------------------------------------------------------------------------
;High priority interrupt routine.

HighIntCode:	
		movff	FSR0H,FSR0H_SHADOW	;save FSR0H register
		movff	FSR0L,FSR0L_SHADOW	;save FSR0L register
		;test other interrupt flags here
;		btfss	PIR2,TMR3IF	;test for Timer3 receive interrupt flag
;		bra		HighInt1	;if RCIF is not set, done with test
;		btfsc	PIE2,TMR3IE	;else test if Timer3 interrupt enabled
;		bra		Timer3ISR		;if so, go get received data
;can do special error handling here - an unexpected interrupt occurred 
HighInt1:	
		reset			;error if no valid interrupt so reset


Main:	
		clrf	LATA	; All outputs low
		clrf	LATB	; All outputs low
		clrf	LATC	; All outputs low
		clrf 	TRISA 	; Config PORTA as all outputs
#ifdef ICD
		movlw	0xE0	
		movwf	TRISB
#else
		clrf 	TRISB 	; Config PORTB as all outputs
#endif
		movlw	0x80	;
		movwf 	TRISC 	; Config PORTC all outputs + tristate for serial rx
		movlw 	0x07	; init adc
		movwf	ADCON1	; init adc
		clrf 	ADCON0  ; poweroff adc
;		clrf 	OutsA;
;		clrf 	OutsB;
;		clrf 	OutsC;

;		MOVLW	0x08
;		CALL	LedGlowOut
;		MOVLW	0x08
;		CALL	LedGlowIn

		lfsr  	FSR0 ,0000h
		lfsr  	FSR1 ,0000h
		
		rcall 	SetupMidi	

		call	MidiInNoteOff
		clrf	TBLPTRU;
		
;		movlw	0x01
;		movwf	MidiByte2;
;		movlw	0x01
;		movwf	MidiByte1;
;		call	MidiInNoteOn
				

MainLoop:	
#ifdef simulate
		nop;	your chance to schedule midi cmd
		movff	MidiTestByte,WREG;
		rcall 	MidiInParser;
		rcall	Timer3ISR
#endif
		call	PollMidiIn;
		bra	MainLoop	;go wait for more data


 org 0x600


MidiInNoteOn: 
		movf	MidiByte2,W;
		bz		MidiInNoteOffNoVelo;		; velocity=0 -> NOTE OFF
MidiInNoteOn2:
		movf	MidiByte1,W;
		addwf	MidiByte1,W;
		movwf	TBLPTRL;
		movlw	0x10
		movwf	TBLPTRH;  page = note number to latch pointer table
		TBLRD*+	; read into TABLAT 
		movf	TABLAT,W

		btfss	WREG,0
		bsf		OUTPUT9		
		btfsc	WREG,0	
		bcf		OUTPUT9		

		btfss	WREG,1
		bsf		OUTPUT10
		btfsc	WREG,1
		bcf		OUTPUT10
		
		btfss	WREG,2
		bsf		OUTPUT11
		btfsc	WREG,2
		bcf		OUTPUT11
		
		btfss	WREG,3
		bsf		OUTPUT12
		btfsc	WREG,3
		bcf		OUTPUT12

		btfss	WREG,4
		bsf		OUTPUT13
		btfsc	WREG,4
		bcf		OUTPUT13

		btfss	WREG,5
		bsf		OUTPUT14
		btfsc	WREG,5
		bcf		OUTPUT14

		btfss	WREG,6
		bsf		OUTPUT15
		btfsc	WREG,6
		bcf		OUTPUT15

		btfss	WREG,7
		bsf		OUTPUT16
		btfsc	WREG,7
		bcf		OUTPUT16

		TBLRD*+	; read into TABLAT 
		movf	TABLAT,W

		btfss	WREG,0
		bsf		OUTPUT1		
		btfsc	WREG,0	
		bcf		OUTPUT1		

		btfss	WREG,1
		bsf		OUTPUT2
		btfsc	WREG,1
		bcf		OUTPUT2
		
		btfss	WREG,2
		bsf		OUTPUT3
		btfsc	WREG,2
		bcf		OUTPUT3
		
		btfss	WREG,3
		bsf		OUTPUT4
		btfsc	WREG,3
		bcf		OUTPUT4

		btfss	WREG,4
		bsf		OUTPUT5
		btfsc	WREG,4
		bcf		OUTPUT5

		btfss	WREG,5
		bsf		OUTPUT6
		btfsc	WREG,5
		bcf		OUTPUT6

		btfss	WREG,6
		bsf		OUTPUT7
		btfsc	WREG,6
		bcf		OUTPUT7

		btfss	WREG,7
		bsf		OUTPUT8
		btfsc	WREG,7
		bcf		OUTPUT8

	return

MidiInNoteOffNoVelo:
MidiInNoteOff:
		clrf MidiByte1
		bra MidiInNoteOn2
	return

MidiInCtrlChange:
		movlw	0x7B; 			AllNotesOff
		subwf	MidiByte1,0;
		bz		MidiInAllNotesOff;

		movlw	.65; 			CC relais
		subwf	MidiByte1,0;
		bz		MidiInRelais
	return

MidiInRelais:
		movf	MidiByte2,W;
		addlw	0
		
		bz		MidiInRelaisOff;		; velocity=0 -> NOTE OFF
		bsf		LATC,2
	return
MidiInRelaisOff:
		bcf		LATC,2
	return

MidiInAllNotesOff:
		clrf MidiByte1
		bra MidiInNoteOn
	return

MidiInProgramChange:
	RETURN		

MidiInSysExStart:
MidiInSysExByte:
MidiInSysExEnd:
MidiInRTReset:
MidiInRTActiveSense:
MidiInRTStop:
MidiInRTContinue:
MidiInRTStart:
MidiInRTTick:
MidiInRTClock:
MidiInTuneRequest:
MidiInAfterTouch:
MidiInChannelPressure:
MidiInPitchWheel:
	RETURN

;	ORG	0x0600
